from setuptools import setup, find_packages

setup(
    name="aalgoi",
    version="1.1.0",
    description="Artificial Algorithm Intelligence with RL - Self-Adaptive Algorithm Selection",
    packages=find_packages(include=["core*", "algorithms*", "interface*", "domains*", "utils*", "aalgoi*"]),
    py_modules=["pipeline"],
    install_requires=[
        "numpy>=1.20.0",
        "networkx>=3.0",
        "psutil>=5.8.0",
        "scikit-learn>=1.0.0",
        "scipy>=1.7.0",
        "click>=8.1.0",
    ],
    python_requires=">=3.8",
)
